@@ -40,6 +40,15 @@ class BaseCommand(api.Command):
             if check_error:
                 raise
 
+    @classmethod
+    def set_column(cls, row, col, val):
+        setattr(row, col, idlutils.db_replace_record(val))
+
+    @classmethod
+    def set_columns(cls, row, **columns):
+        for col, val in columns.items():
+            cls.set_column(row, col, val)
+
     def post_commit(self, txn):
         pass
 
@@ -70,8 +79,7 @@ class DbCreateCommand(BaseCommand):
 
     def run_idl(self, txn):
         row = txn.insert(self.api._tables[self.table])
-        for col, val in self.columns.items():
-            setattr(row, col, idlutils.db_replace_record(val))
+        self.set_columns(row, **self.columns)
         # This is a temporary row to be used within the transaction
         self.result = row
 
@@ -112,7 +120,7 @@ class DbSetCommand(BaseCommand):
                 existing = getattr(record, col, {})
                 existing.update(val)
                 val = existing
-            setattr(record, col, idlutils.db_replace_record(val))
+            self.set_column(record, col, val)
 
 
 class DbAddCommand(BaseCommand):
@@ -146,7 +154,7 @@ class DbAddCommand(BaseCommand):
                     field = getattr(record, self.column, [])
                     field.append(value)
             record.verify(self.column)
-            setattr(record, self.column, idlutils.db_replace_record(field))
+            self.set_column(record, self.column, field)
 
 
 class DbClearCommand(BaseCommand):
@@ -194,8 +202,12 @@ class DbListCommand(BaseCommand):
 
     def run_idl(self, txn):
         table_schema = self.api._tables[self.table]
+        idx = idlutils.get_index_column(table_schema)
         columns = self.columns or list(table_schema.columns.keys()) + ['_uuid']
-        if self.records:
+        # If there's an index for this table, we'll fetch all columns and
+        # remove the unwanted ones based on self.records. Otherwise, let's try
+        # to get the uuid of the wanted ones which is an O(n^2) operation.
+        if not idx and self.records:
             rows = []
             for record in self.records:
                 try:
@@ -208,14 +220,41 @@ class DbListCommand(BaseCommand):
                     raise
         else:
             rows = table_schema.rows.values()
+
+        def _match(row):
+            elem = getattr(row, idx)
+            found = elem in self.records
+            if found:
+                records_found.remove(elem)
+            return found
+
+        records_found = []
+        if idx and self.records:
+            if self.if_exists:
+                match = lambda row: getattr(row, idx) in self.records
+            else:
+                # If we're using the approach of removing the unwanted
+                # elements, we'll use a helper list to remove elements as we
+                # find them in the DB contents. This will help us identify
+                # quickly if there's some record missing to raise a RowNotFound
+                # exception later.
+                records_found = list(self.records)
+                match = _match
+        else:
+            match = lambda row: True
+
         self.result = [
             rowview.RowView(row) if self.row else {
                 c: idlutils.get_column_value(row, c)
                 for c in columns
             }
-            for row in rows
+            for row in rows if match(row)
         ]
 
+        if records_found:
+            raise idlutils.RowNotFound(table=self.table, col=idx,
+                                       match=records_found[0])
+
 
 class DbFindCommand(BaseCommand):
     def __init__(self, api, table, *conditions, **kwargs):
