@@ -91,9 +91,23 @@ class Transaction(api.Transaction):
             status = txn.commit_block()
             if status == txn.TRY_AGAIN:
                 LOG.debug("OVSDB transaction returned TRY_AGAIN, retrying")
+                # In the case that there is a reconnection after
+                # Connection.run() calls self.idl.run() but before do_commit()
+                # is called, commit_block() can loop w/o calling idl.run()
+                # which does the reconnect logic. It will then always return
+                # TRY_AGAIN until we time out and Connection.run() calls
+                # idl.run() again. So, call idl.run() here just in case.
+                self.api.idl.run()
                 continue
-            elif status == txn.ERROR:
-                msg = "OVSDB Error: %s" % txn.get_error()
+            elif status in (txn.ERROR, txn.NOT_LOCKED):
+                msg = 'OVSDB Error: '
+                if status == txn.NOT_LOCKED:
+                    msg += ("The transaction failed because the IDL has "
+                            "been configured to require a database lock "
+                            "but didn't get it yet or has already lost it")
+                else:
+                    msg += txn.get_error()
+
                 if self.log_errors:
                     LOG.error(msg)
                 if self.check_error:
@@ -107,6 +121,8 @@ class Transaction(api.Transaction):
                 LOG.debug("Transaction caused no change")
             elif status == txn.SUCCESS:
                 self.post_commit(txn)
+            else:
+                LOG.debug("Transaction returned an unknown status: %s", status)
 
             return [cmd.result for cmd in self.commands]
 
